// *******************************************************************
// File    : FlatDateTimePicker.cs 
// Author  : Igor Tolmachev
// Email   : info@itsamples.com
// Web     : www.itsamples.com 
// You are free to use/modify this code but leave this header intact.
// *******************************************************************

using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Runtime.InteropServices;

namespace ExifDataChanger
{
	public class FlatDateTimePicker: DateTimePicker
	{	
		private static HatchBrush m_brDotted = new HatchBrush(HatchStyle.SmallCheckerBoard, SystemColors.ControlDark, SystemColors.Window);
		
		private Pen  m_penFocused     = new Pen(m_brDotted, 1);
		private Pen  m_penUsual       = new Pen(SystemColors.ControlDark, 1);
		private Pen  m_penMasked      = new Pen(SystemColors.Window, 2);
		private bool m_bIsDroppedDown = false;

		public FlatDateTimePicker()	: base()
		{
			this.SetStyle(ControlStyles.DoubleBuffer, true);
			this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
		}
		
		protected override void OnValueChanged(EventArgs eventargs)
		{
			base.OnValueChanged (eventargs);
			this.Invalidate();
		}

		protected override void WndProc(ref Message m)
		{
			IntPtr hDC = Win32.GetWindowDC(m.HWnd);
			Graphics g = Graphics.FromHdc(hDC);
			switch (m.Msg)
			{
				case Win32.WM_NC_PAINT:	
					Win32.SendMessage(this.Handle, Win32.WM_ERASEBKGND, hDC, IntPtr.Zero);
					PrintClientArea();
					Win32.SendMessage(this.Handle, Win32.WM_PAINT, IntPtr.Zero, IntPtr.Zero);
					DrawBorders(g);
					m.Result = (IntPtr) 1;	// indicate msg has been processed
					break;
				case Win32.WM_PAINT:	
					base.WndProc(ref m);
					DrawBorders(g);
					DrawButton(g);
					break;
				case Win32.WM_NC_HITTEST: 
					base.WndProc(ref m);
					if(m_bIsDroppedDown)
						this.Invalidate(this.ClientRectangle, false);
					break;
				default:
					base.WndProc(ref m);
					break;
			}
			
			Win32.ReleaseDC(m.HWnd, hDC);
			g.Dispose();				
		}

		private void PrintClientArea()
		{
			Graphics gClient = this.CreateGraphics();
			IntPtr ptrClientDC = gClient.GetHdc();
			Win32.SendMessage(this.Handle, Win32.WM_PRINTCLIENT, ptrClientDC, (IntPtr)0);
			gClient.ReleaseHdc(ptrClientDC);
			gClient.Dispose();
		}

		private void DrawButton(Graphics g)
		{
			Rectangle rect = new Rectangle(this.Width-18, 1, 17, this.Height-2);
			ControlPaint.DrawComboButton(g, rect, ButtonState.Flat);
		}

		private void DrawBorders(Graphics g)
		{
			Rectangle rcMask   = new Rectangle(1, 1, this.Width-2, this.Height-2);
			Rectangle rcClient = new Rectangle(1, 1, this.Width-3, this.Height-3);
			g.DrawRectangle(m_penMasked, rcMask);
			if(this.Focused == false || this.Enabled == false )
				g.DrawRectangle(m_penUsual, rcClient);
			else
				g.DrawRectangle(m_penFocused, rcClient);
		}

		protected override void OnDropDown(EventArgs eventargs)
		{
			m_bIsDroppedDown = true;
			base.OnDropDown(eventargs);
		}
		
		protected override void OnCloseUp(EventArgs eventargs)
		{
			m_bIsDroppedDown = false;
			base.OnCloseUp(eventargs);
		}
	
		protected override void OnLostFocus(System.EventArgs e)
		{
			base.OnLostFocus(e);
			this.Invalidate();
		}

		protected override void OnGotFocus(System.EventArgs e)
		{
			base.OnGotFocus(e);
			this.Invalidate();
		}		
	}
}
